import bpy
import os
import math
import re

bl_info = {
 "name": "Unity Batch FBX Export",
 "description": "Batch export of selected (or all) objects to FBX files for Unity",
 "author": "Stefan Persson",
 "blender": (2, 7, 9),
 "version": (1, 0, 0),
 "category": "Unity",
 "location": "View3D > Tools > Unity Batch FBX Export",
 "warning": "",
 "wiki_url": "",
 "tracker_url": "",
}    

class InterfaceVars(bpy.types.PropertyGroup):
    method = bpy.props.EnumProperty(
        items=[
            ('as_is', 'As Is', 'This exports all selected objects "as is" without any rotation changes/fixes.', '', 0),
            ('rotated_copy', 'Rotated Copy', 'This creates a temporary copies that are rotation fixed for Unity - the copies are deleted after export.', '', 1),
            ('apply_transform', 'Apply Transform', 'This uses Blenders experimental "Apply Transform" export option to fix rotation for Unity - warning, this is experimental and should not be used for animated objects or objects with armature.', '', 2),
        ],
        default='rotated_copy'
    )
    selection = bpy.props.EnumProperty(
        items=[
            ('selected_objects', 'Selected Objects', 'Exports selected objects to FBX files using object names as file names.', '', 0),
            ('all_objects', 'All Objects', 'Exports all mesh objects in scene to FBX files using object names as file names.', '', 1),            
        ],
        default='all_objects'
    )

    
class BatchFBXExportPanel(bpy.types.Panel):
    bl_idname = "ube.batch_export_panel"
    bl_label = "Unity Batch FBX Exporter"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'
    bl_context = "objectmode"    
    bl_options = {'DEFAULT_CLOSED'}
    
    def draw(self, context):

        layout = self.layout

        row = layout.row(align=True)                
        row.label("BATCH EXPORT:")
        row = layout.row(align=True)                
        row.prop(context.scene, 'ueb_batch_export_path')        
        row = layout.row(align=True)
        row.prop(context.window_manager.interface_vars, 'method','Method')
        #row.prop(context.window_manager.interface_vars, 'method', expand=True)
        row = layout.row(align=True)
        row.prop(context.window_manager.interface_vars, 'selection','Selection')
        row = layout.row(align=True)        
        row.operator("ube.batch_export", text="Batch FBX Export", icon='EXPORT')
        
        row = layout.row(align=True)        
        row.label("ROTATION FIX:")
        row = layout.row(align=True)
        row.operator("ube.fix_rotation", text="Fix Rotation (Selected Objects)", icon='ROTATE')
    

class BatchFBXExport(bpy.types.Operator):
    bl_idname = "ube.batch_export"
    bl_label = "Batch FBX Export"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "This exports selected (or all) objects as FBX files with correct scale and rotation when imported into Unity. If you use 'As Is' option you must manually use the 'Fix Rotation' button below for objects first for the rotation to be correct."

    def execute(self, context):        
        basedir = os.path.dirname(bpy.data.filepath)
        if not basedir:
            raise Exception("Blend file is not saved")

        if context.scene.ueb_batch_export_path == "":
            raise Exception("Export path not set")
                      
        org_sel = bpy.context.selected_objects
            
        if (context.window_manager.interface_vars.selection == 'all_objects'):
            for obj in bpy.data.objects:
                if obj.type == 'MESH':
                    obj.select = True
            
        selection = bpy.context.selected_objects
        
        dir = os.path.dirname(bpy.path.abspath(context.scene.ueb_batch_export_path))

        for obj in selection:            
            bpy.ops.object.select_all(action='DESELECT')
            obj.select = True
            
            bpy.context.scene.preloc = bpy.context.selected_objects[0].location		            
            name = bpy.path.clean_name(obj.name)            
            fn = os.path.join(dir, name)

            if (context.window_manager.interface_vars.method == 'as_is'):
                print ("Exporting FBX files using 'As Is' method")
                bpy.context.selected_objects[0].location = (0.0, 0.0, 0.0)
                bpy.ops.export_scene.fbx(filepath=fn + ".fbx", use_selection=True, axis_forward='-Z', axis_up='Y', apply_scale_options='FBX_SCALE_UNITS')
                bpy.context.selected_objects[0].location = bpy.context.scene.preloc                
                
            if (context.window_manager.interface_vars.method == 'rotated_copy'):
                print ("Exporting FBX files using 'Rotated Copy' method")
                obj.name = obj.name + "_original"
                obj_dup = obj.copy()      
                obj_dup.data = obj.data.copy()           
                obj_dup.name = re.sub('_original.001', '', obj_dup.name)
                bpy.data.scenes[0].objects.link(obj_dup)            
                bpy.ops.object.select_all(action='DESELECT')            
                obj_dup.select = True
                bpy.context.selected_objects[0].location = (0.0, 0.0, 0.0)
                bpy.ops.object.transform_apply(rotation = True)
                bpy.ops.transform.rotate(value = -1.57079632679, axis = (1, 0, 0), constraint_axis = (True, False, False), constraint_orientation = 'GLOBAL')
                bpy.ops.object.transform_apply(rotation = True)
                bpy.ops.transform.rotate(value = 1.57079632679, axis = (1, 0, 0), constraint_axis = (True, False, False), constraint_orientation = 'GLOBAL')
                bpy.ops.export_scene.fbx(filepath=fn + ".fbx", use_selection=True, axis_forward='-Z', axis_up='Y', apply_scale_options='FBX_SCALE_UNITS')                
                bpy.ops.object.delete()
                obj.name = re.sub('_original', '', obj.name)
                
            if (context.window_manager.interface_vars.method == 'apply_transform'):
                print ("Exporting FBX files using 'Apply Transform' method")
                bpy.context.selected_objects[0].location = (0.0, 0.0, 0.0)
                bpy.ops.export_scene.fbx(filepath=fn + ".fbx", use_selection=True, axis_forward='-Z', axis_up='Y', bake_space_transform=True)
                bpy.context.selected_objects[0].location = bpy.context.scene.preloc                            
        
        bpy.ops.object.select_all(action='DESELECT')      
        for obj in org_sel:
            obj.select = True
            
        ShowMessageBox("Finished exporting " + str(len(selection)) + " FBX file(s).", "Finished")         
        return {'FINISHED'}
        
class FixRotation(bpy.types.Operator):
    bl_idname = "ube.fix_rotation"
    bl_label = "Fix Rotation"
    bl_options = {'REGISTER', 'UNDO'}
    bl_description = "This rotates selected objects by 90 degrees around the X axis so they align with Unity's Z axis being forward and Y axis being up when exported."

    def execute(self, context):               
        selection = bpy.context.selected_objects               
        for obj in selection:
            if obj.data.users > 1:
                raise Exception("Object \"" + obj.name + "\" has more than one user. Select object and press 'U' to remove 'Object & Data' users and try again.")
        
        for obj in selection:            
            bpy.ops.object.select_all(action='DESELECT')
            obj.select = True
            bpy.ops.object.transform_apply(rotation = True)
            bpy.ops.transform.rotate(value = -1.57079632679, axis = (1, 0, 0), constraint_axis = (True, False, False), constraint_orientation = 'GLOBAL')
            bpy.ops.object.transform_apply(rotation = True)
            bpy.ops.transform.rotate(value = 1.57079632679, axis = (1, 0, 0), constraint_axis = (True, False, False), constraint_orientation = 'GLOBAL')
        
        bpy.ops.object.select_all(action='DESELECT')      
        
        for obj in selection:
            obj.select = True
            
        ShowMessageBox("Finished fixing rotation for  " + str(len(selection)) + " selected object(s).", "Finished")         
        return {'FINISHED'}        
        
def ShowMessageBox(message = "", title = "Message Box", icon = 'INFO'):
    def draw(self, context):
        self.layout.label(message)
    bpy.context.window_manager.popup_menu(draw, title = title, icon = icon)
        
def add_to_menu(self, context) :  
    self.layout.operator("mesh.dropdownexample", icon = "PLUGIN")  
        
def register():
    bpy.types.Scene.ueb_batch_export_path = bpy.props.StringProperty (
        name="Export Path",
        default="",
        description="Define the path where to export",
        subtype='DIR_PATH'
    )
    bpy.types.Scene.preloc = bpy.props.FloatVectorProperty (
        name="preloc",
        default=(0,0,0)
    )
    bpy.utils.register_class(BatchFBXExportPanel) 
    bpy.utils.register_class(BatchFBXExport) 
    bpy.utils.register_class(FixRotation) 
    bpy.utils.register_class(InterfaceVars)
    bpy.types.WindowManager.interface_vars = bpy.props.PointerProperty(type=InterfaceVars)


def unregister():
    del bpy.types.Scene.ueb_batch_export_path
    del bpy.types.Scene.preloc
    bpy.utils.unregister_class(BatchFBXExportPanel) 
    bpy.utils.unregister_class(BatchFBXExport)
    bpy.utils.unregister_class(FixRotation)
    del bpy.types.WindowManager.interface_vars
    bpy.utils.unregister_class(InterfaceVars)
    
if __name__ == "__main__":
    register()

    